@@ -1,4 +1,4 @@
-# Copyright 2016-2021 Nick Boultbee
+# Copyright 2016-2022 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,11 +19,12 @@ class ArtworkUrlCover(CoverSourcePlugin, HTTPDownloadMixin):
     PLUGIN_NAME = _("Artwork URL Cover Source")
     PLUGIN_DESC_MARKUP = _(
         "Downloads covers linked to by the <tt>artwork_url</tt> tag. "
-        "This works with the Soundcloud browser.")
+        "This works with the Soundcloud and Podcasts browsers."
+    )
 
     @classmethod
     def group_by(cls, song):
-        return song.get('album', None)
+        return song.get("album", None)
 
     @staticmethod
     def priority():
@@ -34,12 +35,14 @@ class ArtworkUrlCover(CoverSourcePlugin, HTTPDownloadMixin):
         url = self.url
         if url:
             return path.join(cover_dir, escape_filename(url))
+        return None
 
     @property
     def url(self):
-        return self.song.get('artwork_url', None)
+        return self.song.get("artwork_url", None)
 
     def fetch_cover(self):
         if not self.url:
-            return self.fail('artwork_url missing')
-        self.download(Soup.Message.new('GET', self.url))
+            return self.fail("artwork_url missing")
+        self.download(Soup.Message.new("GET", self.url))
+        return None
