@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 # Copyright 2005-2009 Joe Wreschnig, Steven Robertson
-#           2012-2018 Nick Boultbee
+#           2012-2023 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,37 +17,36 @@ from quodlibet.plugins.events import EventPlugin
 from quodlibet import util
 from quodlibet.util import print_d
 from quodlibet.browsers.playlists import PlaylistsBrowser
-try:
-    from quodlibet.qltk import notif, Icons
-except Exception:
-    notif = None
+from quodlibet.qltk import notif, Icons
 
 
 class RandomAlbum(EventPlugin):
-    PLUGIN_ID = 'Random Album Playback'
-    PLUGIN_NAME = _('Random Album Playback')
-    PLUGIN_DESC = _("Starts a random album when your playlist reaches its "
-                    "end. It requires that your active browser supports "
-                    "filtering by album.")
+    PLUGIN_ID = "Random Album Playback"
+    PLUGIN_NAME = _("Random Album Playback")
+    PLUGIN_DESC = _(
+        "Starts a random album when your playlist reaches its "
+        "end. It requires that your active browser supports "
+        "filtering by album."
+    )
     PLUGIN_ICON = Icons.MEDIA_SKIP_FORWARD
 
-    weights = {}
+    weights: dict[str, float] = {}
     use_weights = False
     # Not a dict because we want to impose a particular order
     # Third item is to specify a non-default aggregation function
     keys = [
-                ("rating", _("Rated higher"), None),
-                ("playcount", _("Played more often"), 'avg'),
-                ("skipcount", _("Skipped more often"), 'avg'),
-                ("lastplayed", _("Played more recently"), None),
-                ("laststarted", _("Started more recently"), None),
-                ("added", _("Added more recently"), None),
-                ("length", _("Longer albums"), None),
-            ]
+        ("rating", _("Rated higher"), None),
+        ("playcount", _("Played more often"), "avg"),
+        ("skipcount", _("Skipped more often"), "avg"),
+        ("lastplayed", _("Played more recently"), None),
+        ("laststarted", _("Started more recently"), None),
+        ("added", _("Added more recently"), None),
+        ("length", _("Longer albums"), None),
+    ]
 
     def __init__(self):
-        for (key, text, func) in self.keys:
-            val = config.getfloat("plugins", "randomalbum_%s" % key, 0.0)
+        for key, _text, _func in self.keys:
+            val = config.getfloat("plugins", f"randomalbum_{key}", 0.0)
             self.weights[key] = val
 
         use = config.getint("plugins", "randomalbum_use_weights", 0)
@@ -60,7 +58,7 @@ class RandomAlbum(EventPlugin):
         def changed_cb(hscale, key):
             val = hscale.get_value()
             self.weights[key] = val
-            config.set("plugins", "randomalbum_%s" % key, val)
+            config.set("plugins", f"randomalbum_{key}", val)
 
         def delay_changed_cb(spin):
             self.delay = int(spin.get_value())
@@ -70,8 +68,7 @@ class RandomAlbum(EventPlugin):
             self.use_weights = check.get_active()
             for w in widgets:
                 w.set_sensitive(self.use_weights)
-            config.set("plugins", "randomalbum_use_weights",
-                    str(int(self.use_weights)))
+            config.set("plugins", "randomalbum_use_weights", str(int(self.use_weights)))
 
         vbox = Gtk.VBox(spacing=12)
         table = Gtk.Table(n_rows=len(self.keys) + 1, n_columns=3)
@@ -79,7 +76,8 @@ class RandomAlbum(EventPlugin):
 
         hbox = Gtk.HBox(spacing=6)
         spin = Gtk.SpinButton(
-            adjustment=Gtk.Adjustment.new(self.delay, 0, 3600, 1, 10, 0))
+            adjustment=Gtk.Adjustment.new(self.delay, 0, 3600, 1, 10, 0)
+        )
         spin.connect("value-changed", delay_changed_cb)
         hbox.pack_start(spin, False, True, 0)
         lbl = Gtk.Label(label=_("seconds before starting next album"))
@@ -100,33 +98,36 @@ class RandomAlbum(EventPlugin):
 
         # Less label
         less_lbl = Gtk.Label()
-        arr = Gtk.Arrow(arrow_type=Gtk.ArrowType.LEFT,
-                        shadow_type=Gtk.ShadowType.OUT)
-        less_lbl.set_markup("<i>%s</i>" % util.escape(_("avoid")))
+        arr = Gtk.Arrow(arrow_type=Gtk.ArrowType.LEFT, shadow_type=Gtk.ShadowType.OUT)
+        less_lbl.set_markup(util.italic(_("avoid")))
         less_lbl.set_alignment(0, 0)
         hb = Gtk.HBox(spacing=0)
         hb.pack_start(arr, False, True, 0)
         hb.pack_start(less_lbl, True, True, 0)
-        table.attach(hb, 1, 2, 0, 1, xpadding=3,
-                     xoptions=Gtk.AttachOptions.FILL)
+        table.attach(hb, 1, 2, 0, 1, xpadding=3, xoptions=Gtk.AttachOptions.FILL)
         # More label
         more_lbl = Gtk.Label()
-        arr = Gtk.Arrow(arrow_type=Gtk.ArrowType.RIGHT,
-                        shadow_type=Gtk.ShadowType.OUT)
-        more_lbl.set_markup("<i>%s</i>" % util.escape(_("prefer")))
+        arr = Gtk.Arrow(arrow_type=Gtk.ArrowType.RIGHT, shadow_type=Gtk.ShadowType.OUT)
+        more_lbl.set_markup(util.italic(_("prefer")))
         more_lbl.set_alignment(1, 0)
         hb = Gtk.HBox(spacing=0)
         hb.pack_end(arr, False, True, 0)
         hb.pack_end(more_lbl, True, True, 0)
-        table.attach(hb, 2, 3, 0, 1, xpadding=3,
-                     xoptions=Gtk.AttachOptions.FILL)
+        table.attach(hb, 2, 3, 0, 1, xpadding=3, xoptions=Gtk.AttachOptions.FILL)
 
-        for (idx, (key, text, func)) in enumerate(self.keys):
+        for idx, (key, text, _func) in enumerate(self.keys):
             lbl = Gtk.Label(label=text)
             lbl.set_alignment(0, 0)
-            table.attach(lbl, 0, 1, idx + 1, idx + 2,
-                         xoptions=Gtk.AttachOptions.FILL,
-                         xpadding=3, ypadding=3)
+            table.attach(
+                lbl,
+                0,
+                1,
+                idx + 1,
+                idx + 2,
+                xoptions=Gtk.AttachOptions.FILL,
+                xpadding=3,
+                ypadding=3,
+            )
             adj = Gtk.Adjustment(lower=-1.0, upper=1.0, step_increment=0.1)
             hscale = Gtk.HScale(adjustment=adj)
             hscale.set_value(self.weights[key])
@@ -134,8 +135,7 @@ class RandomAlbum(EventPlugin):
             hscale.set_show_fill_level(False)
             hscale.connect("value-changed", changed_cb, key)
             lbl.set_mnemonic_widget(hscale)
-            table.attach(hscale, 1, 3, idx + 1, idx + 2,
-                         xpadding=3, ypadding=3)
+            table.attach(hscale, 1, 3, idx + 1, idx + 2, xpadding=3, ypadding=3)
 
         return vbox
 
@@ -147,16 +147,14 @@ class RandomAlbum(EventPlugin):
         # based on normalized means, and also normalizes the scale of each
         # weight slider in the prefs pane.
         ranked = {}
-        for (tag, text, func) in self.keys:
-            tag_key = ("~#%s:%s" % (tag, func) if func
-                       else "~#%s" % tag)
-            ranked[tag] = sorted(albums,
-                                 key=lambda al: al.get(tag_key))
+        for tag, _text, func in self.keys:
+            tag_key = f"~#{tag}:{func}" if func else f"~#{tag}"
+            ranked[tag] = sorted(albums, key=lambda al: al.get(tag_key))
 
         scores = {}
         for album in albums:
             scores[album] = 0
-            for (tag, text, func) in self.keys:
+            for tag, _text, _func in self.keys:
                 rank = ranked[tag].index(album)
                 scores[album] += rank * self.weights[tag]
 
@@ -168,7 +166,7 @@ class RandomAlbum(EventPlugin):
             browser = app.window.browser
 
             if self.disabled_for_browser(browser):
-                print_d("%s doesn't support album filtering" % browser.name)
+                print_d(f"{browser.name} doesn't support album filtering")
                 return
 
             albumlib = app.library.albums
@@ -193,14 +191,12 @@ class RandomAlbum(EventPlugin):
                 album_scores = self._score(chosen_albums)
                 # Find highest score value
                 max_score = max(album_scores, key=lambda t: t[0])[0]
-                print_d("Maximum score found: %0.1f" % max_score)
+                print_d(f"Maximum score found: {max_score:0.1f}")
                 # Filter albums by highest score value
-                albums = [(sc, al)
-                          for sc, al in album_scores
-                          if sc == max_score]
+                albums = [(sc, al) for sc, al in album_scores if sc == max_score]
                 print_d("Albums with maximum score:")
-                for score, album in albums:
-                    print_d("  %s" % album("album"))
+                for _score, album in albums:
+                    print_d("  {}".format(album("album")))
 
                 # Pick random album from list of highest scored albums
                 album = random.choice(albums)[1]
@@ -208,26 +204,25 @@ class RandomAlbum(EventPlugin):
                 album = random.choice(values)
 
             if album is not None:
-                print_d("Chosen album: %s" % album("album"))
+                print_d("Chosen album: {}".format(album("album")))
                 self.schedule_change(album)
 
     def schedule_change(self, album):
         if self.delay:
-            srcid = GLib.timeout_add(1000 * self.delay,
-                                     self.change_album, album)
-            if notif is None:
-                return
-            task = notif.Task(_("Random Album"),
-                              _("Waiting to start %s") %
-                                    util.bold(util.escape(album("album"))),
-                              stop=lambda: GLib.source_remove(srcid))
+            srcid = GLib.timeout_add(1000 * self.delay, self.change_album, album)
+            task = notif.Task(
+                _("Random Album"),
+                _("Waiting to start %s") % util.bold(album("album")),
+                stop=lambda: GLib.source_remove(srcid),
+            )
 
             def countdown():
                 for i in range(10 * self.delay):
-                    task.update(i / (10. * self.delay))
+                    task.update(i / (10.0 * self.delay))
                     yield True
                 task.finish()
                 yield False
+
             GLib.timeout_add(100, next, countdown())
         else:
             self.change_album(album)
@@ -240,7 +235,7 @@ class RandomAlbum(EventPlugin):
         if browser.can_filter_albums():
             browser.filter_albums([album.key])
         else:
-            browser.filter('album', [album("album")])
+            browser.filter("album", [album("album")])
         GLib.idle_add(self.unpause)
 
     def unpause(self):
@@ -254,5 +249,4 @@ class RandomAlbum(EventPlugin):
             app.player.paused = True
 
     def disabled_for_browser(self, browser):
-        return (not browser.can_filter("album") or
-                isinstance(browser, PlaylistsBrowser))
+        return not browser.can_filter("album") or isinstance(browser, PlaylistsBrowser)
