@@ -1,7 +1,7 @@
 #
 #    Duplicates songs plugin.
 #
-#    Copyright (C) 2011-2019 Nick Boultbee
+#    Copyright (C) 2011-2023 Nick Boultbee
 #
 #    Finds "duplicates" of songs selected by searching the library for
 #    others with the same user-configurable "key", presenting a browser-like
@@ -50,13 +50,11 @@ class DuplicateSongsView(RCMHintedTreeView):
                 selected.append(row[0])
         return selected
 
-    def Menu(self, library):
+    def menu(self, library):
         songs = self.get_selected_songs()
         if not songs:
-            return
-
-        menu = SongsMenu(
-            library, songs, delete=True, plugins=False, playlists=False)
+            return None
+        menu = SongsMenu(library, songs, delete=True, plugins=False, playlists=False)
         menu.show_all()
         return menu
 
@@ -79,11 +77,11 @@ class DuplicateSongsView(RCMHintedTreeView):
             row = model.find_row(song)
             if row:
                 group_row = model.iter_parent(row.iter)
-                print_d("Found parent group = %s" % group_row)
+                print_d(f"Found parent group = {group_row}")
                 model.remove(row.iter)
                 num_kids = model.iter_n_children(group_row)
                 if num_kids < Duplicates.MIN_GROUP_SIZE:
-                    print_d("Removing group %s" % group_row)
+                    print_d(f"Removing group {group_row}")
                     model.remove(group_row)
             else:
                 pass
@@ -106,39 +104,37 @@ class DuplicateSongsView(RCMHintedTreeView):
             key = Duplicates.get_key(song)
             row = model.find_row(song)
             if row:
-                print_d("Changed duplicated file \"%s\" (Row=%s)" %
-                        (song("~artist~title"), row))
+                print_d(
+                    f"Changed duplicated file {song('~artist~title')!r} (Row={row})"
+                )
                 parent = model.iter_parent(row.iter)
                 old_key = model[parent][0]
                 if old_key != key:
-                    print_d("Key changed from \"%s\" -> \"%s\"" %
-                            (old_key, key))
+                    print_d(f'Key changed from "{old_key}" -> "{key}"')
                     self._removed(library, [song])
                     self._added(library, [song])
                 else:
                     # Still might be a displayable change
-                    print_d("Calling model.row_changed(%s, %s)..." %
-                            (row.path, row.iter))
+                    print_d(f"Calling model.row_changed({row.path}, {row.iter})...")
                     model.row_changed(row.path, row.iter)
             else:
                 model.add_to_existing_group(key, song)
 
     def __init__(self, model):
-        super(DuplicateSongsView, self).__init__(model)
-        connect_obj(self, 'row-activated',
-                            self.__select_song, app.player)
+        super().__init__(model)
+        connect_obj(self, "row-activated", self.__select_song, app.player)
         # Selecting multiple is a nice feature it turns out.
         self.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
 
         # Handle signals propagated from the underlying library
         self.connected_library_sigs = []
-        SIGNAL_MAP = {
-            'removed': self._removed,
-            'added': self._added,
-            'changed': self._changed
+        signal_map = {
+            "removed": self._removed,
+            "added": self._added,
+            "changed": self._changed,
         }
-        for (sig, callback) in SIGNAL_MAP.items():
-            print_d("Listening to library.%s signals" % sig)
+        for sig, callback in signal_map.items():
+            print_d(f"Listening to library.{sig} signals")
             connect_destroy(app.library, sig, callback)
 
 
@@ -146,13 +142,19 @@ class DuplicatesTreeModel(Gtk.TreeStore):
     """A tree store to model duplicated song information"""
 
     # Define columns to display (and how, in lieu of using qltk.browsers)
+    @staticmethod
     def i(x):
         return x
+
     TAG_MAP = [
-        ("artist", i), ("title", i), ("album", i),
+        ("artist", i),
+        ("title", i),
+        ("album", i),
         ("~#length", lambda s: util.format_time_display(int(s))),
-        ("~#filesize", lambda s: util.format_size(int(s))), ("~#bitrate", i),
-        ("~filename", i)]
+        ("~#filesize", lambda s: util.format_size(int(s))),
+        ("~#bitrate", i),
+        ("~filename", i),
+    ]
     # Now make a dict. This seems clunky.
     tag_functions = {}
     for t, f in TAG_MAP:
@@ -186,8 +188,7 @@ class DuplicatesTreeModel(Gtk.TreeStore):
         return None
 
     def add_to_existing_group(self, key, song):
-        """Tries to add a song to an existing group. Returns None if not able
-        """
+        """Tries to add a song to an existing group. Returns None if not able"""
         for parent in self:
             if key == parent[0]:
                 print_d("Found group", self)
@@ -198,16 +199,17 @@ class DuplicatesTreeModel(Gtk.TreeStore):
     @classmethod
     def __make_row(cls, song):
         """Construct GTK row for a song, with all columns"""
-        return [song] + [util.escape(str(f(song.comma(tag)))) for
-                         (tag, f) in cls.TAG_MAP]
+        return [song] + [
+            util.escape(str(f(song.comma(tag)))) for (tag, f) in cls.TAG_MAP
+        ]
 
     def add_group(self, key, songs):
         """Adds a new group, returning the row created"""
         group = AudioFileGroup(songs, real_keys_only=False)
         # Add the group first.
-        parent = self.append(None,
-            [key] +
-            [self.group_value(group, tag) for tag, f in self.TAG_MAP])
+        parent = self.append(
+            None, [key] + [self.group_value(group, tag) for tag, f in self.TAG_MAP]
+        )
 
         for s in songs:
             self.append(parent, self.__make_row(s))
@@ -224,7 +226,7 @@ class DuplicatesTreeModel(Gtk.TreeStore):
     def remove(self, itr):
         if self.__iter and self[itr].path == self[self.__iter].path:
             self.__iter = None
-        super(DuplicatesTreeModel, self).remove(itr)
+        super().remove(itr)
 
     def get(self):
         return [row[0] for row in self]
@@ -233,43 +235,38 @@ class DuplicatesTreeModel(Gtk.TreeStore):
     def get_current(self):
         if self.__iter is None:
             return None
-        elif self.is_empty():
+        if self.is_empty():
             return None
-        else:
-            return self[self.__iter][0]
+        return self[self.__iter][0]
 
     @property
     def get_current_path(self):
         if self.__iter is None:
             return None
-        elif self.is_empty():
+        if self.is_empty():
             return None
-        else:
-            return self[self.__iter].path
+        return self[self.__iter].path
 
     @property
     def get_current_iter(self):
         if self.__iter is None:
             return None
-        elif self.is_empty():
+        if self.is_empty():
             return None
-        else:
-            return self.__iter
+        return self.__iter
 
     def is_empty(self):
         return not len(self)
 
     def __init__(self):
-        super(DuplicatesTreeModel, self).__init__(
-            object, str, str, str, str, str, str, str)
+        super().__init__(object, str, str, str, str, str, str, str)
 
 
 class DuplicateDialog(Gtk.Window):
     """Main dialog for browsing duplicate results"""
 
     def __quit(self, widget=None, response=None):
-        if response == Gtk.ResponseType.OK or \
-                response == Gtk.ResponseType.CLOSE:
+        if response == Gtk.ResponseType.OK or response == Gtk.ResponseType.CLOSE:
             print_d("Exiting plugin on user request...")
         self.finished = True
         self.destroy()
@@ -277,18 +274,18 @@ class DuplicateDialog(Gtk.Window):
 
     def __songs_popup_menu(self, songlist):
         path, col = songlist.get_cursor()
-        menu = songlist.Menu(app.library)
+        menu = songlist.menu(app.library)
         if menu is not None:
             return songlist.popup_menu(menu, 0, Gtk.get_current_event_time())
+        return None
 
     def __init__(self, model):
-        songs_text = numeric_phrase("%d duplicate group",
-                                    "%d duplicate groups",
-                                    len(model))
-        super(DuplicateDialog, self).__init__()
+        songs_text = numeric_phrase(
+            "%d duplicate group", "%d duplicate groups", len(model)
+        )
+        super().__init__()
         self.set_destroy_with_parent(True)
-        self.set_title("Quod Libet - %s (%s)" % (Duplicates.PLUGIN_NAME,
-                                                 songs_text))
+        self.set_title(f"Quod Libet - {Duplicates.PLUGIN_NAME} ({songs_text})")
         self.finished = False
         self.set_default_size(960, 480)
         self.set_border_width(6)
@@ -304,9 +301,12 @@ class DuplicateDialog(Gtk.Window):
             cell.set_property("markup", text)
 
         # Set up the columns
-        for i, (tag, f) in enumerate(DuplicatesTreeModel.TAG_MAP):
-            e = (Pango.EllipsizeMode.START if tag == '~filename'
-                else Pango.EllipsizeMode.END)
+        for i, (tag, _f) in enumerate(DuplicatesTreeModel.TAG_MAP):
+            e = (
+                Pango.EllipsizeMode.START
+                if tag == "~filename"
+                else Pango.EllipsizeMode.END
+            )
             render = Gtk.CellRendererText()
             render.set_property("ellipsize", e)
             col = Gtk.TreeViewColumn(util.tag(tag), render)
@@ -321,7 +321,7 @@ class DuplicateDialog(Gtk.Window):
             col.set_cell_data_func(render, cell_text, i + 1)
             view.append_column(col)
 
-        view.connect('popup-menu', self.__songs_popup_menu)
+        view.connect("popup-menu", self.__songs_popup_menu)
         swin.add(view)
         # A basic information area
         hbox = Gtk.HBox(spacing=6)
@@ -339,11 +339,13 @@ class DuplicateDialog(Gtk.Window):
         connect_obj(expand, "clicked", expand_all, view)
         hbox.pack_start(expand, False, True, 0)
 
-        label = Gtk.Label(label=_("Duplicate key expression is '%s'") %
-                Duplicates.get_key_expression())
+        label = Gtk.Label(
+            label=_("Duplicate key expression is '%s'")
+            % Duplicates.get_key_expression()
+        )
         hbox.pack_start(label, True, True, 0)
         close = Button(_("_Close"), Icons.WINDOW_CLOSE)
-        close.connect('clicked', self.__quit)
+        close.connect("clicked", self.__quit)
         hbox.pack_start(close, False, True, 0)
 
         vbox = Gtk.VBox(spacing=6)
@@ -354,31 +356,31 @@ class DuplicateDialog(Gtk.Window):
 
 
 class Duplicates(SongsMenuPlugin, PluginConfigMixin):
-    PLUGIN_ID = 'Duplicates'
-    PLUGIN_NAME = _('Duplicates Browser')
-    PLUGIN_DESC = _('Finds and displays similarly tagged versions of songs.')
+    PLUGIN_ID = "Duplicates"
+    PLUGIN_NAME = _("Duplicates Browser")
+    PLUGIN_DESC = _("Finds and displays similarly tagged versions of songs.")
     PLUGIN_ICON = Icons.EDIT_SELECT_ALL
 
     MIN_GROUP_SIZE = 2
     _CFG_KEY_KEY = "key_expression"
     __DEFAULT_KEY_VALUE = "~artist~title~version"
 
-    _CFG_REMOVE_WHITESPACE = 'remove_whitespace'
-    _CFG_REMOVE_DIACRITICS = 'remove_diacritics'
-    _CFG_REMOVE_PUNCTUATION = 'remove_punctuation'
-    _CFG_CASE_INSENSITIVE = 'case_insensitive'
+    _CFG_REMOVE_WHITESPACE = "remove_whitespace"
+    _CFG_REMOVE_DIACRITICS = "remove_diacritics"
+    _CFG_REMOVE_PUNCTUATION = "remove_punctuation"
+    _CFG_CASE_INSENSITIVE = "case_insensitive"
 
     plugin_handles = any_song(is_finite)
 
     # Cached values
     key_expression = None
-    __cfg_cache = {}
 
     @classmethod
     def get_key_expression(cls):
         if not cls.key_expression:
-            cls.key_expression = (
-                    cls.config_get(cls._CFG_KEY_KEY, cls.__DEFAULT_KEY_VALUE))
+            cls.key_expression = cls.config_get(
+                cls._CFG_KEY_KEY, cls.__DEFAULT_KEY_VALUE
+            )
         return cls.key_expression
 
     @classmethod
@@ -394,8 +396,13 @@ class Duplicates(SongsMenuPlugin, PluginConfigMixin):
         e = UndoEntry()
         e.set_text(cls.get_key_expression())
         e.connect("changed", key_changed)
-        e.set_tooltip_markup(_("Accepts QL tag expressions like "
-                "<tt>~artist~title</tt> or <tt>musicbrainz_track_id</tt>"))
+        e.set_tooltip_markup(
+            _(
+                "Accepts QL tag expressions like "
+                "<tt>~artist~title</tt> or "
+                "<tt>musicbrainz_track_id</tt>"
+            )
+        )
         lbl = Gtk.Label(label=_("_Group duplicates by:"))
         lbl.set_mnemonic_widget(e)
         lbl.set_use_underline(True)
@@ -413,7 +420,7 @@ class Duplicates(SongsMenuPlugin, PluginConfigMixin):
         ]
         vb2 = Gtk.VBox(spacing=6)
         for key, label in toggles:
-            ccb = ConfigCheckButton(label, 'plugins', cls._config_key(key))
+            ccb = ConfigCheckButton(label, "plugins", cls._config_key(key))
             ccb.set_active(cls.config_get_bool(key))
             vb2.pack_start(ccb, True, True, 0)
 
@@ -438,7 +445,6 @@ class Duplicates(SongsMenuPlugin, PluginConfigMixin):
 
     def plugin_songs(self, songs):
         model = DuplicatesTreeModel()
-        self.__cfg_cache = {}
 
         # Index all songs by our custom key
         # TODO: make this cache-friendly
@@ -447,7 +453,7 @@ class Duplicates(SongsMenuPlugin, PluginConfigMixin):
         for song in songs:
             key = self.get_key(song)
             if key and key in groups:
-                print_d("Found duplicate based on '%s'" % key)
+                print_d(f"Found duplicate based on '{key}'")
                 groups[key].add(song._song)
             elif key:
                 groups[key] = {song._song}
@@ -458,7 +464,7 @@ class Duplicates(SongsMenuPlugin, PluginConfigMixin):
                 groups[key].add(song)
 
         # Now display the grouped duplicates
-        for (key, children) in groups.items():
+        for key, children in groups.items():
             if len(children) < self.MIN_GROUP_SIZE:
                 continue
             # The parent (group) label
