@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman
 #
 # This program is free software; you can redistribute it and/or modify
@@ -6,12 +5,13 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
+
 from senf import fsnative, path2fsn
 
 from ._audio import AudioFile
 
 
-extensions = []
+extensions: list[str] = []
 
 
 class RemoteFile(AudioFile):
@@ -27,9 +27,8 @@ class RemoteFile(AudioFile):
 
     def __getitem__(self, key):
         # we used to save them with the wrong type
-        value = super(RemoteFile, self).__getitem__(key)
-        if key in ("~filename", "~mountpoint") and \
-                not isinstance(value, fsnative):
+        value = super().__getitem__(key)
+        if key in ("~filename", "~mountpoint") and not isinstance(value, fsnative):
             value = path2fsn(value)
 
         return value
@@ -55,12 +54,12 @@ class RemoteFile(AudioFile):
     def can_change(self, k=None):
         if k is None:
             return []
-        else:
-            return False
+        return False
 
     @property
     def key(self):
         return self["~uri"]
 
+
 loader = RemoteFile
 types = [RemoteFile]
