@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2014 Christoph Reiter
 #           2018 Ludovic Druette
 #
@@ -52,11 +51,12 @@ def find_active_backend():
     print_d("Trying to find a mmkeys backend")
     for backend in iter_backends():
         if backend.is_active():
-            print_d("Found %r" % backend.__name__)
+            print_d(f"Found {backend.__name__!r}")
             return backend
+    return None
 
 
-class MMKeysHandler(object):
+class MMKeysHandler:
     """Manages multiple keybinding backends and translates the generated
     events to actions on the player backend.
     """
@@ -90,7 +90,7 @@ class MMKeysHandler(object):
             self._backend.grab()
 
     def _callback(self, action):
-        print_d("Event %r from %r" % (action, type(self._backend).__name__))
+        print_d(f"Event {action!r} from {type(self._backend).__name__!r}")
 
         def seek_relative(seconds):
             current = player.get_position()
