@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2012,2014 Christoph Reiter
 #           2014,2017 Nick Boultbee
 #
@@ -71,12 +70,12 @@ class Dialog(Gtk.Dialog):
     def __init__(self, *args, **kwargs):
         if not should_use_header_bar():
             kwargs.pop("use_header_bar", None)
-        super(Dialog, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def get_titlebar(self):
         try:
             # gtk+ >=3.16
-            return super(Dialog, self).get_titlebar()
+            return super().get_titlebar()
         except AttributeError:
             return None
 
@@ -89,7 +88,7 @@ class Dialog(Gtk.Dialog):
             # content height make the window a bit taller.
             if height != -1:
                 height += 20
-        super(Dialog, self).set_default_size(width, height)
+        super().set_default_size(width, height)
 
     def add_icon_button(self, label, icon_name, response_id):
         """Like add_button() but allows to pass an icon name"""
@@ -110,30 +109,36 @@ class Window(Gtk.Window):
     ^W will close the window.
     """
 
-    windows = []
+    windows: list[Gtk.Window] = []
     _preven_inital_show = False
 
     def __init__(self, *args, **kwargs):
         self._header_bar = None
         dialog = kwargs.pop("dialog", True)
-        super(Window, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         type(self).windows.append(self)
         if dialog:
+            # Modal is the only way to center the window on the parent
+            # with wayland atm
+            if is_wayland():
+                self.set_modal(True)
             self.set_type_hint(Gdk.WindowTypeHint.DIALOG)
         self.set_destroy_with_parent(True)
         self.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
-        connect_obj(self, 'destroy', type(self).windows.remove, self)
-        self.connect('key-press-event', self._on_key_press)
+        connect_obj(self, "destroy", type(self).windows.remove, self)
+        self.connect("key-press-event", self._on_key_press)
 
     def _on_key_press(self, widget, event):
-        is_dialog = (self.get_type_hint() == Gdk.WindowTypeHint.DIALOG)
+        is_dialog = self.get_type_hint() == Gdk.WindowTypeHint.DIALOG
 
         if (is_dialog and is_accel(event, "Escape")) or (
-                not is_dialog and is_accel(event, "<Primary>w")):
+            not is_dialog and is_accel(event, "<Primary>w")
+        ):
             # Do not close the window if we edit a Gtk.CellRendererText.
             # Focus the treeview instead.
-            if isinstance(self.get_focus(), Gtk.Entry) and \
-                isinstance(self.get_focus().get_parent(), Gtk.TreeView):
+            if isinstance(self.get_focus(), Gtk.Entry) and isinstance(
+                self.get_focus().get_parent(), Gtk.TreeView
+            ):
                 self.get_focus().get_parent().grab_focus()
                 return Gdk.EVENT_PROPAGATE
             self.close()
@@ -165,7 +170,7 @@ class Window(Gtk.Window):
     def set_default_size(self, width, height):
         if self._header_bar:
             width, height = fix_default_size(width, height)
-        super(Window, self).set_default_size(width, height)
+        super().set_default_size(width, height)
 
     def use_header_bar(self):
         """Try to use a headerbar, returns the widget or None in case
@@ -205,16 +210,6 @@ class Window(Gtk.Window):
         if self._header_bar is not None:
             return self._header_bar.get_show_close_button()
 
-        screen = Gdk.Screen.get_default()
-        if hasattr(screen, "get_window_manager_name"):
-            # X11 only
-            wm_name = screen.get_window_manager_name()
-            # Older Gnome Shell didn't show close buttons.
-            # We can't get the version but the GTK+ version is a good guess,
-            # I guess..
-            if wm_name == "GNOME Shell" and gtk_version < (3, 18):
-                return False
-
         return True
 
     def present(self):
@@ -226,14 +221,14 @@ class Window(Gtk.Window):
         try:
             from gi.repository import GdkX11
         except ImportError:
-            super(Window, self).present()
+            super().present()
         else:
             window = self.get_window()
             if window and isinstance(window, GdkX11.X11Window):
                 timestamp = GdkX11.x11_get_server_time(window)
                 self.present_with_time(timestamp)
             else:
-                super(Window, self).present()
+                super().present()
 
     def set_transient_for(self, parent):
         """Set a parent for the window.
@@ -246,10 +241,11 @@ class Window(Gtk.Window):
 
         if parent is None or not is_toplevel:
             if parent:
-                print_w("Not a toplevel window set for: %r" % self)
+                print_w(f"Not a toplevel window set for: {self!r}")
             from quodlibet import app
+
             parent = app.window
-        super(Window, self).set_transient_for(parent)
+        super().set_transient_for(parent)
 
     @classmethod
     def prevent_inital_show(cls, value):
@@ -267,7 +263,7 @@ class Window(Gtk.Window):
         return not self._preven_inital_show
 
 
-class PersistentWindowMixin(object):
+class PersistentWindowMixin:
     """A mixin for saving/restoring window size/position/maximized state"""
 
     def enable_window_tracking(self, config_prefix, size_suffix=""):
@@ -288,14 +284,14 @@ class PersistentWindowMixin(object):
         self.__name = config_prefix
         self.__size_suffix = size_suffix
         self.__save_size_pos_deferred = DeferredSignal(
-            self.__do_save_size_pos, timeout=50, owner=self)
-        self.connect('configure-event', self.__configure_event)
-        self.connect('window-state-event', self.__window_state_changed)
-        self.connect('notify::visible', self.__visible_changed)
+            self.__do_save_size_pos, timeout=50, owner=self
+        )
+        self.connect("configure-event", self.__configure_event)
+        self.connect("window-state-event", self.__window_state_changed)
+        self.connect("notify::visible", self.__visible_changed)
         parent = self.get_transient_for()
         if parent:
-            connect_destroy(
-                parent, 'configure-event', self.__parent_configure_event)
+            connect_destroy(parent, "configure-event", self.__parent_configure_event)
         self.__restore_window_state()
 
     def __visible_changed(self, *args):
@@ -314,7 +310,7 @@ class PersistentWindowMixin(object):
     def __conf(self, name):
         if name == "size":
             name += "_" + self.__size_suffix
-        return "%s_%s" % (self.__name, name)
+        return f"{self.__name}_{name}"
 
     def __restore_state(self):
         print_d("Restore state")
@@ -325,7 +321,7 @@ class PersistentWindowMixin(object):
 
     def __restore_position(self):
         print_d("Restore position")
-        pos = config.get('memory', self.__conf("position"), "")
+        pos = config.get("memory", self.__conf("position"), "")
         if not pos:
             return
 
@@ -344,7 +340,7 @@ class PersistentWindowMixin(object):
 
     def __restore_size(self):
         print_d("Restore size")
-        value = config.get('memory', self.__conf("size"), "")
+        value = config.get("memory", self.__conf("size"), "")
         if not value:
             return
 
@@ -377,9 +373,9 @@ class PersistentWindowMixin(object):
     def _should_ignore_state(self):
         if self.__state & Gdk.WindowState.MAXIMIZED:
             return True
-        elif self.__state & Gdk.WindowState.FULLSCREEN:
+        if self.__state & Gdk.WindowState.FULLSCREEN:
             return True
-        elif not self.get_visible():
+        if not self.get_visible():
             return True
         return False
 
@@ -404,8 +400,8 @@ class PersistentWindowMixin(object):
             x -= px
             y -= py
 
-        pos_value = '%s %s' % (x, y)
-        config.set('memory', self.__conf("position"), pos_value)
+        pos_value = f"{x} {y}"
+        config.set("memory", self.__conf("position"), pos_value)
 
     def __window_state_changed(self, window, event):
         self.__state = event.new_window_state
@@ -415,15 +411,15 @@ class PersistentWindowMixin(object):
         config.set("memory", self.__conf("maximized"), maximized)
 
 
-class _Unique(object):
-    """A mixin for window-like classes to ensure one instance per class. """
+class _Unique:
+    """A mixin for window-like classes to ensure one instance per class."""
 
     __window = None
 
-    def __new__(klass, *args, **kwargs):
-        window = klass.__window
+    def __new__(cls, *args, **kwargs):
+        window = cls.__window
         if window is None:
-            return super(_Unique, klass).__new__(klass, *args, **kwargs)
+            return super().__new__(cls, *args, **kwargs)
         # Look for widgets in the args, if there is one and it has
         # a new top level window, re-parent and reposition the window.
         widgets = [w for w in args if isinstance(w, Gtk.Widget)]
@@ -438,17 +434,16 @@ class _Unique(object):
         return window
 
     @classmethod
-    def is_not_unique(klass):
+    def is_not_unique(cls):
         """Returns True if a window instance already exists."""
-        return bool(klass.__window)
+        return bool(cls.__window)
 
     def __init__(self, *args, **kwargs):
         if type(self).__window:
             return
-        else:
-            type(self).__window = self
-        super(_Unique, self).__init__(*args, **kwargs)
-        connect_obj(self, 'destroy', self.__destroy, self)
+        type(self).__window = self
+        super().__init__(*args, **kwargs)
+        connect_obj(self, "destroy", self.__destroy, self)
 
     def __destroy(self, *args):
         type(self).__window = None
