@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2006 Joe Wreschnig, Alexandre Passos
 #           2014 Christoph Reiter
 #           2016 Nick Boultbee
@@ -12,24 +11,21 @@
 
 from gi.repository import GLib
 
-from quodlibet.compat import PY2, listkeys
-
-
-class _Routine(object):
 
+class _Routine:
     def __init__(self, pool, func, funcid, priority, timeout, args, kwargs):
         self.priority = priority
         self.timeout = timeout
         self._source_id = None
 
         def wrap(func, funcid, args, kwargs):
-            for value in func(*args, **kwargs):
+            for _value in func(*args, **kwargs):
                 yield True
             pool.remove(funcid)
             yield False
 
         f = wrap(func, funcid, args, kwargs)
-        self.source_func = f.next if PY2 else f.__next__
+        self.source_func = f.__next__
 
     @property
     def paused(self):
@@ -50,10 +46,10 @@ class _Routine(object):
 
         if self.timeout:
             self._source_id = GLib.timeout_add(
-                self.timeout, self.source_func, priority=self.priority)
+                self.timeout, self.source_func, priority=self.priority
+            )
         else:
-            self._source_id = GLib.idle_add(
-                self.source_func, priority=self.priority)
+            self._source_id = GLib.idle_add(self.source_func, priority=self.priority)
 
     def pause(self):
         """Pause, if already paused, do nothing"""
@@ -65,8 +61,7 @@ class _Routine(object):
         self._source_id = None
 
 
-class CoPool(object):
-
+class CoPool:
     def __init__(self):
         self.__routines = {}
 
@@ -102,7 +97,7 @@ class CoPool(object):
     def _get(self, funcid):
         if funcid in self.__routines:
             return self.__routines[funcid]
-        raise ValueError("no pooled routine %r" % funcid)
+        raise ValueError(f"no pooled routine {funcid!r}")
 
     def remove(self, funcid):
         """Stop a registered routine."""
@@ -114,7 +109,7 @@ class CoPool(object):
     def remove_all(self):
         """Stop all running routines."""
 
-        for funcid in listkeys(self.__routines):
+        for funcid in list(self.__routines.keys()):
             self.remove(funcid)
 
     def pause(self, funcid):
