@@ -7,7 +7,6 @@
 
 import locale
 import re
-from typing import List, Dict
 
 from quodlibet import _
 
@@ -22,11 +21,11 @@ class Massager:
     """Massage a tag value from various 'okay' formats to the
     'correct' format."""
 
-    tags: List[str] = []
+    tags: list[str] = []
     error = "Metaerror. This should be overridden in subclasses."
-    options: List[str] = []
+    options: list[str] = []
 
-    _massagers: "Dict[str, Massager]" = {}
+    _massagers: "dict[str, Massager]" = {}
 
     def validate(self, value):
         """Returns a validated value.
@@ -95,7 +94,7 @@ def error_message(tag, value):
     try:
         return Massager.for_tag(tag).error
     except KeyError:
-        return u""
+        return ""
 
 
 def get_options(tag):
@@ -112,10 +111,13 @@ def get_options(tag):
 @Massager._register
 class DateMassager(Massager):
     tags = ["date"]
-    error = _("The date must be entered in 'YYYY', 'YYYY-MM-DD' or "
-              "'YYYY-MM-DD HH:MM:SS' format.")
-    __match = re.compile(r"^\d{4}([-.]\d{2}([-.]\d{2}([T ]\d{2}"
-                         r"([:.]\d{2}([:.]\d{2})?)?)?)?)?$").match
+    error = _(
+        "The date must be entered in 'YYYY', 'YYYY-MM-DD' or "
+        "'YYYY-MM-DD HH:MM:SS' format."
+    )
+    __match = re.compile(
+        r"^\d{4}([-.]\d{2}([-.]\d{2}([T ]\d{2}([:.]\d{2}([:.]\d{2})?)?)?)?)?$"
+    ).match
 
     def validate(self, value):
         value = value.strip().replace(".", "-").replace("/", "-")
@@ -133,16 +135,15 @@ class GainMassager(Massager):
     def validate(self, value):
         if self.__match(value):
             return value
-        else:
+        try:
+            f = float(value.split()[0])
+        except (IndexError, TypeError, ValueError):
             try:
-                f = float(value.split()[0])
-            except (IndexError, TypeError, ValueError):
-                try:
-                    f = locale.atof(value.split()[0])
-                except (IndexError, TypeError, ValueError):
-                    raise ValidationError
-            else:
-                return (u"%+f" % f).rstrip("0") + " dB"
+                f = locale.atof(value.split()[0])
+            except (IndexError, TypeError, ValueError) as e:
+                raise ValidationError from e
+        else:
+            return (f"{f:+f}").rstrip("0") + " dB"
 
 
 @Massager._register
@@ -157,8 +158,8 @@ class PeakMassager(Massager):
         except (TypeError, ValueError):
             try:
                 f = locale.atof(value)
-            except (TypeError, ValueError):
-                raise ValidationError
+            except (TypeError, ValueError) as e:
+                raise ValidationError from e
         else:
             if f < 0 or f >= 2:
                 raise ValidationError
@@ -167,24 +168,29 @@ class PeakMassager(Massager):
 
 @Massager._register
 class MBIDMassager(Massager):
-    tags = ["musicbrainz_trackid", "musicbrainz_albumid",
-            "musicbrainz_artistid", "musicbrainz_albumartistid",
-            "musicbrainz_trmid", "musicip_puid"]
+    tags = [
+        "musicbrainz_trackid",
+        "musicbrainz_albumid",
+        "musicbrainz_artistid",
+        "musicbrainz_albumartistid",
+        "musicbrainz_trmid",
+        "musicip_puid",
+    ]
     error = _("MusicBrainz IDs must be in UUID format.")
 
     def validate(self, value):
-        value = value.encode('ascii', 'replace').decode("ascii")
-        value = u"".join(filter(str.isalnum, value.strip().lower()))
+        value = value.encode("ascii", "replace").decode("ascii")
+        value = "".join(filter(str.isalnum, value.strip().lower()))
         try:
             int(value, 16)
-        except ValueError:
-            raise ValidationError
+        except ValueError as e:
+            raise ValidationError from e
         else:
             if len(value) != 32:
                 raise ValidationError
-            else:
-                return u"-".join([value[:8], value[8:12], value[12:16],
-                                  value[16:20], value[20:]])
+            return "-".join(
+                [value[:8], value[8:12], value[12:16], value[16:20], value[20:]]
+            )
 
 
 @Massager._register
@@ -192,8 +198,9 @@ class MBAlbumStatus(Massager):
     tags = ["musicbrainz_albumstatus"]
     # Translators: Leave "official", "promotional", and "bootleg"
     # untranslated. They are the three possible literal values.
-    error = _("MusicBrainz release status must be 'official', "
-              "'promotional', or 'bootleg'.")
+    error = _(
+        "MusicBrainz release status must be 'official', 'promotional', or 'bootleg'."
+    )
     options = ["official", "promotional", "bootleg"]
 
     def validate(self, value):
@@ -209,8 +216,6 @@ class LanguageMassager(Massager):
 
     options = ISO_639_2
 
-    tags = ["language"]
-
     def validate(self, value):
         # Issue 439: Actually, allow free-text through
         return value
