@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2012,2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -7,7 +6,7 @@
 # (at your option) any later version.
 
 import sys
-import imp
+import importlib
 
 from os.path import dirname
 from traceback import format_exception
@@ -17,8 +16,7 @@ from quodlibet.util.importhelper import get_importables, load_module
 from quodlibet.util import print_d
 
 
-class Module(object):
-
+class Module:
     def __init__(self, name, module, deps, path):
         self.name = name
         self.module = module
@@ -39,18 +37,17 @@ class Module(object):
         return False
 
     def __repr__(self):
-        return "<%s name=%r>" % (type(self).__name__, self.name)
+        return f"<{type(self).__name__} name={self.name!r}>"
 
 
-class ModuleImportError(object):
-
+class ModuleImportError:
     def __init__(self, name, exception, traceback):
         self.name = name
         self.exception = exception
         self.traceback = traceback
 
 
-class ModuleScanner(object):
+class ModuleScanner:
     """
     Handles plugin modules. Takes a list of directories and searches
     for loadable python modules/packages in all of them.
@@ -63,6 +60,7 @@ class ModuleScanner(object):
     modules - A dict of Name: Module for all successfully loaded modules
 
     """
+
     def __init__(self, folders):
         self.__folders = folders
         self.__modules = {}  # name: module
@@ -94,7 +92,7 @@ class ModuleScanner(object):
 
         # get what is there atm
         for folder in self.__folders:
-            for name, path, deps in get_importables(folder, True):
+            for name, path, deps in get_importables(folder):
                 # take the basename as module key, later modules win
                 info[name] = (path, deps)
 
@@ -122,7 +120,7 @@ class ModuleScanner(object):
         self.__failures.clear()
 
         # add new ones
-        for (name, (path, deps)) in info.items():
+        for name, (path, deps) in info.items():
             if name in self.__modules:
                 continue
 
@@ -131,14 +129,13 @@ class ModuleScanner(object):
                 # https://github.com/quodlibet/quodlibet/issues/1093
                 parent = "quodlibet.fake"
                 if parent not in sys.modules:
-                    sys.modules[parent] = imp.new_module(parent)
+                    spec = importlib.machinery.ModuleSpec(parent, None, is_package=True)
+                    sys.modules[parent] = importlib.util.module_from_spec(spec)
                 vars(sys.modules["quodlibet"])["fake"] = sys.modules[parent]
 
-                mod = load_module(name, parent + ".plugins",
-                                  dirname(path), reload=True)
+                mod = load_module(name, parent + ".plugins", dirname(path))
                 if mod is None:
                     continue
-
             except Exception as err:
                 text = format_exception(*sys.exc_info())
                 self.__failures[name] = ModuleImportError(name, err, text)
@@ -146,7 +143,9 @@ class ModuleScanner(object):
                 added.append(name)
                 self.__modules[name] = Module(name, mod, deps, path)
 
-        print_d("Rescanning done: %d added, %d removed, %d error(s)" %
-                (len(added), len(removed), len(self.__failures)))
+        print_d(
+            "Rescanning done: %d added, %d removed, %d error(s)"
+            % (len(added), len(removed), len(self.__failures))
+        )
 
         return removed, added
